@@ -109,6 +109,7 @@ void QCameraPrivate::_q_error(int error, const QString &errorString)
     this->error = QCamera::Error(error);
     this->errorString = errorString;
 
+    emit q->errorOccurred(this->error);
     emit q->error(this->error);
 }
 
@@ -206,12 +207,12 @@ void QCameraPrivate::initControls()
 
         error = QCamera::NoError;
     } else {
-        control = 0;
-        locksControl = 0;
-        deviceControl = 0;
-        infoControl = 0;
-        viewfinderSettingsControl = 0;
-        viewfinderSettingsControl2 = 0;
+        control = nullptr;
+        locksControl = nullptr;
+        deviceControl = nullptr;
+        infoControl = nullptr;
+        viewfinderSettingsControl = nullptr;
+        viewfinderSettingsControl2 = nullptr;
 
         error = QCamera::ServiceMissingError;
         errorString = QCamera::tr("The camera service is missing");
@@ -241,16 +242,16 @@ void QCameraPrivate::clear()
         provider->releaseService(service);
     }
 
-    cameraExposure = 0;
-    cameraFocus = 0;
-    imageProcessing = 0;
-    control = 0;
-    locksControl = 0;
-    deviceControl = 0;
-    infoControl = 0;
-    viewfinderSettingsControl = 0;
-    viewfinderSettingsControl2 = 0;
-    service = 0;
+    cameraExposure = nullptr;
+    cameraFocus = nullptr;
+    imageProcessing = nullptr;
+    control = nullptr;
+    locksControl = nullptr;
+    deviceControl = nullptr;
+    infoControl = nullptr;
+    viewfinderSettingsControl = nullptr;
+    viewfinderSettingsControl2 = nullptr;
+    service = nullptr;
 }
 
 void QCameraPrivate::updateLockStatus()
@@ -328,7 +329,7 @@ QCamera::QCamera(QObject *parent):
     d->init();
 
     // Select the default camera
-    if (d->service != 0 && d->deviceControl)
+    if (d->service != nullptr && d->deviceControl)
         d->deviceControl->setSelectedDevice(d->deviceControl->defaultDevice());
 }
 
@@ -347,18 +348,35 @@ QCamera::QCamera(const QByteArray& deviceName, QObject *parent):
     Q_D(QCamera);
     d->init();
 
-    if (d->service != 0) {
-        //pass device name to service
-        if (d->deviceControl) {
-            const QString name = QString::fromLatin1(deviceName);
-            for (int i = 0; i < d->deviceControl->deviceCount(); i++) {
-                if (d->deviceControl->deviceName(i) == name) {
-                    d->deviceControl->setSelectedDevice(i);
-                    break;
-                }
+    bool found = false;
+    // Pass device name to service.
+    if (d->deviceControl) {
+        const QString name = QString::fromLatin1(deviceName);
+        for (int i = 0; i < d->deviceControl->deviceCount(); i++) {
+            if (d->deviceControl->deviceName(i) == name) {
+                d->deviceControl->setSelectedDevice(i);
+                found = true;
+                break;
             }
         }
     }
+
+    // The camera should not be used if device with requested name does not exist.
+    if (!found) {
+        if (d->service) {
+            if (d->control)
+                d->service->releaseControl(d->control);
+            if (d->deviceControl)
+                d->service->releaseControl(d->deviceControl);
+            if (d->infoControl)
+                d->service->releaseControl(d->infoControl);
+        }
+        d->control = nullptr;
+        d->deviceControl = nullptr;
+        d->infoControl = nullptr;
+        d->error = QCamera::ServiceMissingError;
+        d->errorString = QCamera::tr("The camera service is missing");
+    }
 }
 
 /*!
@@ -368,22 +386,8 @@ QCamera::QCamera(const QByteArray& deviceName, QObject *parent):
 */
 
 QCamera::QCamera(const QCameraInfo &cameraInfo, QObject *parent)
-    : QMediaObject(*new QCameraPrivate,
-                   parent,
-                   QMediaServiceProvider::defaultServiceProvider()->requestService(Q_MEDIASERVICE_CAMERA,
-                                                                                   QMediaServiceProviderHint(cameraInfo.deviceName().toLatin1())))
+    : QCamera(cameraInfo.deviceName().toLatin1(), parent)
 {
-    Q_D(QCamera);
-    d->init();
-
-    if (d->service != 0 && d->deviceControl) {
-        for (int i = 0; i < d->deviceControl->deviceCount(); i++) {
-            if (d->deviceControl->deviceName(i) == cameraInfo.deviceName()) {
-                d->deviceControl->setSelectedDevice(i);
-                break;
-            }
-        }
-    }
 }
 
 /*!
@@ -406,7 +410,7 @@ QCamera::QCamera(QCamera::Position position, QObject *parent)
     Q_D(QCamera);
     d->init();
 
-    if (d->service != 0 && d->deviceControl) {
+    if (d->service != nullptr && d->deviceControl) {
         bool selectDefault = true;
 
         if (d->infoControl && position != UnspecifiedPosition) {
@@ -440,7 +444,7 @@ QCamera::~QCamera()
 QMultimedia::AvailabilityStatus QCamera::availability() const
 {
     Q_D(const QCamera);
-    if (d->control == NULL)
+    if (d->control == nullptr)
         return QMultimedia::ServiceMissing;
 
     if (d->deviceControl && d->deviceControl->deviceCount() == 0)
@@ -494,7 +498,7 @@ void QCamera::setViewfinder(QVideoWidget *viewfinder)
     // We don't know (in this library) that QVideoWidget inherits QObject
     QObject *viewFinderObject = reinterpret_cast<QObject*>(viewfinder);
 
-    d->viewfinder = viewFinderObject && bind(viewFinderObject) ? viewFinderObject : 0;
+    d->viewfinder = viewFinderObject && bind(viewFinderObject) ? viewFinderObject : nullptr;
 }
 
 /*!
@@ -515,7 +519,7 @@ void QCamera::setViewfinder(QGraphicsVideoItem *viewfinder)
     // but QObject inheritance depends on QObject coming first, so try this out.
     QObject *viewFinderObject = reinterpret_cast<QObject*>(viewfinder);
 
-    d->viewfinder = viewFinderObject && bind(viewFinderObject) ? viewFinderObject : 0;
+    d->viewfinder = viewFinderObject && bind(viewFinderObject) ? viewFinderObject : nullptr;
 }
 
 /*!
@@ -535,14 +539,14 @@ void QCamera::setViewfinder(QAbstractVideoSurface *surface)
         if (d->viewfinder)
             unbind(d->viewfinder);
 
-        d->viewfinder = 0;
+        d->viewfinder = nullptr;
 
         if (surface && bind(&d->surfaceViewfinder))
             d->viewfinder = &d->surfaceViewfinder;
     } else if (!surface) {
         //unbind the surfaceViewfinder if null surface is set
         unbind(&d->surfaceViewfinder);
-        d->viewfinder = 0;
+        d->viewfinder = nullptr;
     }
 }
 
@@ -641,7 +645,7 @@ void QCamera::setViewfinderSettings(const QCameraViewfinderSettings &settings)
     If \a settings is non null, the returned list is reduced to settings matching the given partial
     \a settings.
 
-    The camera must be loaded before calling this function, otherwise the returned list
+    The status of the camera must be LoadedStatus before calling this function, otherwise the returned list
     is empty.
 
     \sa setViewfinderSettings(), supportedViewfinderResolutions(), supportedViewfinderFrameRateRanges(),
@@ -820,7 +824,7 @@ void QCamera::setCaptureMode(QCamera::CaptureModes mode)
     Starts the camera.
 
     State is changed to QCamera::ActiveState if camera is started
-    successfully, otherwise error() signal is emitted.
+    successfully, otherwise errorOccurred() signal is emitted.
 
     While the camera state is changed to QCamera::ActiveState,
     starting the camera service can be asynchronous with the actual
@@ -835,6 +839,10 @@ void QCamera::start()
 /*!
     Stops the camera.
     The camera state is changed from QCamera::ActiveState to QCamera::LoadedState.
+
+    In this state, the camera still consumes power.
+
+    \sa unload(), QCamera::UnloadedState
 */
 void QCamera::stop()
 {
@@ -843,15 +851,17 @@ void QCamera::stop()
 }
 
 /*!
-    Open the camera device.
-    The camera state is changed to QCamera::LoadedStatus.
+    Opens the camera device.
+    The camera state is changed to QCamera::LoadedState.
 
-    It's not necessary to explcitly load the camera,
-    unless unless the application have to read the supported camera
-    settings and change the default depending on the camera capabilities.
+    It's not necessary to explicitly load the camera, unless the application
+    needs to read the supported camera settings and change the default values
+    according to the camera capabilities.
 
-    In all the other cases it's possible to start the camera directly
-    from unloaded state.
+    In all the other cases, it's possible to start the camera directly
+    from the unloaded state.
+
+    /sa QCamera::UnloadedState
 */
 void QCamera::load()
 {
@@ -860,8 +870,8 @@ void QCamera::load()
 }
 
 /*!
-    Close the camera device and deallocate the related resources.
-    The camera state is changed to QCamera::UnloadedStatus.
+    Closes the camera device and deallocates the related resources.
+    The camera state is changed to QCamera::UnloadedState.
 */
 void QCamera::unload()
 {
@@ -908,7 +918,7 @@ QCamera::Status QCamera::status() const
 
 
 /*!
-    Returns the lock types, camera supports.
+    Returns the lock types that the camera supports.
 */
 QCamera::LockTypes QCamera::supportedLocks() const
 {
@@ -1075,23 +1085,22 @@ void QCamera::unlock()
 
 /*!
     \enum QCamera::State
-    \value UnloadedState
-           The initial camera state, with camera not loaded,
-           the camera capabilities except of supported capture modes
-           are unknown.
 
+    This enum holds the current state of the camera.
+
+    \value UnloadedState
+           The initial camera state, with camera not loaded.
+           The camera capabilities, except supported capture modes,
+           are unknown.
            While the supported settings are unknown in this state,
            it's allowed to set the camera capture settings like codec,
            resolution, or frame rate.
-
     \value LoadedState
            The camera is loaded and ready to be configured.
-
-           In the Idle state it's allowed to query camera capabilities,
+           In this state it's allowed to query camera capabilities,
            set capture resolution, codecs, etc.
-
            The viewfinder is not active in the loaded state.
-
+           The camera consumes power in the loaded state.
     \value ActiveState
            In the active state as soon as camera is started
            the viewfinder displays video frames and the
@@ -1106,46 +1115,40 @@ void QCamera::unlock()
 
 /*!
     \enum QCamera::Status
+
+    This enum holds the current status of the camera.
+
     \value ActiveStatus
            The camera has been started and can produce data.
            The viewfinder displays video frames in active state.
-
            Depending on backend, changing some camera settings like
            capture mode, codecs or resolution in ActiveState may lead
            to changing the camera status to LoadedStatus and StartingStatus while
            the settings are applied and back to ActiveStatus when the camera is ready.
-
     \value StartingStatus
            The camera is starting in result of state transition to QCamera::ActiveState.
            The camera service is not ready to capture yet.
-
     \value StoppingStatus
            The camera is stopping in result of state transition from QCamera::ActiveState
            to QCamera::LoadedState or QCamera::UnloadedState.
-
     \value StandbyStatus
            The camera is in the power saving standby mode.
            The camera may come to the standby mode after some time of inactivity
            in the QCamera::LoadedState state.
-
     \value LoadedStatus
            The camera is loaded and ready to be configured.
            This status indicates the camera device is opened and
            it's possible to query for supported image and video capture settings,
            like resolution, framerate and codecs.
-
     \value LoadingStatus
            The camera device loading in result of state transition from
            QCamera::UnloadedState to QCamera::LoadedState or QCamera::ActiveState.
-
     \value UnloadingStatus
            The camera device is unloading in result of state transition from
            QCamera::LoadedState or QCamera::ActiveState to QCamera::UnloadedState.
-
     \value UnloadedStatus
            The initial camera status, with camera not loaded.
            The camera capabilities including supported capture settings may be unknown.
-
     \value UnavailableStatus
            The camera or camera backend is not available.
 */
@@ -1159,6 +1162,9 @@ void QCamera::unlock()
 
 /*!
     \enum QCamera::CaptureMode
+
+    This enum holds the capture mode of the camera.
+
     \value CaptureViewfinder Camera is only configured to display viewfinder.
     \value CaptureStillImage Camera is configured for still frames capture.
     \value CaptureVideo  Camera is configured for video capture.
@@ -1167,6 +1173,8 @@ void QCamera::unlock()
 /*!
     \enum QCamera::LockType
 
+    This enum holds the camera lock type.
+
     \value NoLock
     \value LockExposure
         Lock camera exposure.
@@ -1207,28 +1215,32 @@ void QCamera::unlock()
 
 /*!
   \enum QCamera::LockStatus
+
+    This enum holds the overall status for all the requested camera locks.
+
     \value Unlocked
         The application is not interested in camera settings value.
         The camera may keep this parameter without changes, this is common with camera focus,
         or adjust exposure and white balance constantly to keep the viewfinder image nice.
-
     \value Searching
         The application has requested the camera focus, exposure or white balance lock with
-        QCamera::searchAndLock(). This state indicates the camera is focusing or calculating exposure and white balance.
-
+        QCamera::searchAndLock(). This state indicates the camera is focusing or
+        calculating exposure and white balance.
     \value Locked
         The camera focus, exposure or white balance is locked.
-        The camera is ready to capture, application may check the exposure parameters.
-
-        The locked state usually means the requested parameter stays the same,
-        except of the cases when the parameter is requested to be constantly updated.
-        For example in continuous focusing mode, the focus is considered locked as long
-        and the object is in focus, even while the actual focusing distance may be constantly changing.
+        The camera is ready to capture, application may check the exposure
+        stays the same, parameters. The \c Locked status usually means the
+        requested parameter except in the cases when the parameter is requested
+        to be constantly updated. For example, in continuous focusing mode,
+        the focus is considered locked as long as the object is in focus, even
+        while the actual focusing distance may be constantly changing.
 */
 
 /*!
     \enum QCamera::LockChangeReason
 
+    This enum holds the reason why the camera lock status changed.
+
     \value UserRequest
         The lock status changed in result of user request, usually to unlock camera settings.
     \value LockAcquired
@@ -1249,6 +1261,8 @@ void QCamera::unlock()
 /*!
     \enum QCamera::Error
 
+    This enum holds the last error code.
+
     \value  NoError      No errors have occurred.
     \value  CameraError  An error has occurred.
     \value  InvalidRequestError System resource doesn't support requested functionality.
@@ -1258,6 +1272,14 @@ void QCamera::unlock()
 
 /*!
     \fn void QCamera::error(QCamera::Error value)
+    \obsolete
+
+    Use errorOccurred() instead.
+*/
+
+/*!
+    \fn void QCamera::errorOccurred(QCamera::Error value)
+    \since 5.15
 
     Signal emitted when error state changes to \a value.
 */
