@@ -77,7 +77,7 @@ QVideoWidgetControlBackend::QVideoWidgetControlBackend(
     connect(control, SIGNAL(fullScreenChanged(bool)), widget, SLOT(_q_fullScreenChanged(bool)));
 
     QBoxLayout *layout = new QVBoxLayout;
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
 
     QWidget *videoWidget = control->videoWidget();
@@ -145,7 +145,8 @@ QRendererVideoWidgetBackend::QRendererVideoWidgetBackend(
     connect(m_surface, SIGNAL(surfaceFormatChanged(QVideoSurfaceFormat)),
             this, SLOT(formatChanged(QVideoSurfaceFormat)));
 
-    m_rendererControl->setSurface(m_surface);
+    if (m_rendererControl)
+        m_rendererControl->setSurface(m_surface);
 }
 
 QRendererVideoWidgetBackend::~QRendererVideoWidgetBackend()
@@ -153,14 +154,21 @@ QRendererVideoWidgetBackend::~QRendererVideoWidgetBackend()
     delete m_surface;
 }
 
+QAbstractVideoSurface *QRendererVideoWidgetBackend::videoSurface() const
+{
+    return m_surface;
+}
+
 void QRendererVideoWidgetBackend::releaseControl()
 {
-    m_service->releaseControl(m_rendererControl);
+    if (m_service && m_rendererControl)
+        m_service->releaseControl(m_rendererControl);
 }
 
 void QRendererVideoWidgetBackend::clearSurface()
 {
-    m_rendererControl->setSurface(0);
+    if (m_rendererControl)
+        m_rendererControl->setSurface(0);
 }
 
 void QRendererVideoWidgetBackend::setBrightness(int brightness)
@@ -220,7 +228,6 @@ void QRendererVideoWidgetBackend::hideEvent(QHideEvent *)
 {
 #if QT_CONFIG(opengl)
     m_updatePaintDevice = true;
-    m_surface->setGLContext(0);
 #endif
 }
 
@@ -257,7 +264,7 @@ void QRendererVideoWidgetBackend::paintEvent(QPaintEvent *event)
                 || painter.paintEngine()->type() == QPaintEngine::OpenGL2)) {
             m_updatePaintDevice = false;
 
-            m_surface->setGLContext(const_cast<QGLContext *>(QGLContext::currentContext()));
+            m_surface->updateGLContext();
             if (m_surface->supportedShaderTypes() & QPainterVideoSurface::GlslShader) {
                 m_surface->setShaderType(QPainterVideoSurface::GlslShader);
             } else {
@@ -469,7 +476,7 @@ void QVideoWidgetPrivate::clearService()
 
             delete rendererBackend;
             rendererBackend = 0;
-        } else {
+        } else if (windowBackend) {
             windowBackend->releaseControl();
 
             delete windowBackend;
@@ -515,18 +522,15 @@ bool QVideoWidgetPrivate::createWindowBackend()
 
 bool QVideoWidgetPrivate::createRendererBackend()
 {
-    if (QMediaControl *control = service->requestControl(QVideoRendererControl_iid)) {
-        if (QVideoRendererControl *rendererControl = qobject_cast<QVideoRendererControl *>(control)) {
-            rendererBackend = new QRendererVideoWidgetBackend(service, rendererControl, q_func());
-            currentBackend = rendererBackend;
+    QMediaControl *control = service
+                           ? service->requestControl(QVideoRendererControl_iid)
+                           : nullptr;
+    rendererBackend = new QRendererVideoWidgetBackend(service,
+        qobject_cast<QVideoRendererControl *>(control), q_func());
+    currentBackend = rendererBackend;
+    setCurrentControl(rendererBackend);
 
-            setCurrentControl(rendererBackend);
-
-            return true;
-        }
-        service->releaseControl(control);
-    }
-    return false;
+    return !service || (service && control);
 }
 
 void QVideoWidgetPrivate::_q_serviceDestroyed()
@@ -611,7 +615,7 @@ void QVideoWidgetPrivate::_q_dimensionsChanged()
     The \a parent is passed to QWidget.
 */
 QVideoWidget::QVideoWidget(QWidget *parent)
-    : QWidget(parent, 0)
+    : QWidget(parent, {})
     , d_ptr(new QVideoWidgetPrivate)
 {
     d_ptr->q_ptr = this;
@@ -621,13 +625,13 @@ QVideoWidget::QVideoWidget(QWidget *parent)
   \internal
 */
 QVideoWidget::QVideoWidget(QVideoWidgetPrivate &dd, QWidget *parent)
-    : QWidget(parent, 0)
+    : QWidget(parent, {})
     , d_ptr(&dd)
 {
     d_ptr->q_ptr = this;
 
     QPalette palette = QWidget::palette();
-    palette.setColor(QPalette::Background, Qt::black);
+    palette.setColor(QPalette::Window, Qt::black);
     setPalette(palette);
 }
 
@@ -695,6 +699,29 @@ bool QVideoWidget::setMediaObject(QMediaObject *object)
     return true;
 }
 
+/*!
+    \since 5.15
+    \property QVideoWidget::videoSurface
+    \brief Returns the underlaying video surface that can render video frames
+    to the current widget.
+    This property is never \c nullptr.
+    Example of how to render video frames to QVideoWidget:
+    \snippet multimedia-snippets/video.cpp Widget Surface
+    \sa QMediaPlayer::setVideoOutput
+*/
+
+QAbstractVideoSurface *QVideoWidget::videoSurface() const
+{
+    auto d = const_cast<QVideoWidgetPrivate *>(d_func());
+
+    if (!d->rendererBackend) {
+        d->clearService();
+        d->createRendererBackend();
+    }
+
+    return d->rendererBackend->videoSurface();
+}
+
 /*!
     \property QVideoWidget::aspectRatioMode
     \brief how video is scaled with respect to its aspect ratio.
@@ -1007,7 +1034,11 @@ void QVideoWidget::paintEvent(QPaintEvent *event)
 }
 
 #if defined(Q_OS_WIN)
+#  if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+bool QVideoWidget::nativeEvent(const QByteArray &eventType, void *message, qintptr *result)
+#  else
 bool QVideoWidget::nativeEvent(const QByteArray &eventType, void *message, long *result)
+#  endif
 {
     Q_D(QVideoWidget);
     Q_UNUSED(eventType);
@@ -1023,7 +1054,7 @@ bool QVideoWidget::nativeEvent(const QByteArray &eventType, void *message, long
 }
 #endif
 
-#include "moc_qvideowidget.cpp"
-#include "moc_qvideowidget_p.cpp"
 QT_END_NAMESPACE
 
+#include "moc_qvideowidget.cpp"
+#include "moc_qvideowidget_p.cpp"
