@@ -188,19 +188,30 @@ void QAndroidCaptureSession::setState(QMediaRecorder::State state)
         start();
         break;
     case QMediaRecorder::PausedState:
-        // Not supported by Android API
-        qWarning("QMediaRecorder::PausedState is not supported on Android");
+        pause();
         break;
     }
 }
 
 void QAndroidCaptureSession::start()
 {
-    if (m_state == QMediaRecorder::RecordingState || m_status != QMediaRecorder::LoadedStatus)
+    if (m_state == QMediaRecorder::RecordingState
+        || (m_status != QMediaRecorder::LoadedStatus && m_status != QMediaRecorder::PausedStatus))
         return;
 
     setStatus(QMediaRecorder::StartingStatus);
 
+    if (m_state == QMediaRecorder::PausedState) {
+        if (!m_mediaRecorder || !m_mediaRecorder->resume()) {
+            emit error(QMediaRecorder::FormatError, QLatin1String("Unable to resume the media recorder."));
+            if (m_cameraSession)
+                restartViewfinder();
+        } else {
+            updateStartState();
+        }
+        return;
+    }
+
     if (m_mediaRecorder) {
         m_mediaRecorder->release();
         delete m_mediaRecorder;
@@ -289,7 +300,11 @@ void QAndroidCaptureSession::start()
             restartViewfinder();
         return;
     }
+    updateStartState();
+}
 
+void QAndroidCaptureSession::updateStartState()
+{
     m_elapsedTime.start();
     m_notifyTimer.start();
     updateDuration();
@@ -302,9 +317,20 @@ void QAndroidCaptureSession::start()
         m_cameraSession->camera()->setupPreviewFrameCallback();
     }
 
+    QMediaRecorder::State oldState = m_state;
+    QMediaRecorder::Status oldStatus = m_status;
+
     m_state = QMediaRecorder::RecordingState;
-    emit stateChanged(m_state);
-    setStatus(QMediaRecorder::RecordingStatus);
+    m_status = QMediaRecorder::RecordingStatus;
+
+    m_actualOutputLocation = m_usedOutputLocation;
+    emit actualLocationChanged(m_actualOutputLocation);
+
+    if (m_state != oldState)
+        emit stateChanged(m_state);
+
+    if (m_status != oldStatus)
+        emit statusChanged(m_status);
 }
 
 void QAndroidCaptureSession::stop(bool error)
@@ -317,6 +343,7 @@ void QAndroidCaptureSession::stop(bool error)
     m_mediaRecorder->stop();
     m_notifyTimer.stop();
     updateDuration();
+    m_previousElapsedTime = 0;
     m_elapsedTime.invalidate();
     m_mediaRecorder->release();
     delete m_mediaRecorder;
@@ -347,6 +374,23 @@ void QAndroidCaptureSession::stop(bool error)
         setStatus(QMediaRecorder::LoadedStatus);
 }
 
+void QAndroidCaptureSession::pause()
+{
+    if (m_state == QMediaRecorder::PausedState || m_mediaRecorder == 0)
+        return;
+
+    setStatus(QMediaRecorder::PausedStatus);
+
+    m_mediaRecorder->pause();
+    m_notifyTimer.stop();
+    updateDuration();
+    m_previousElapsedTime = m_duration;
+    m_elapsedTime.invalidate();
+
+    m_state = QMediaRecorder::PausedState;
+    emit stateChanged(m_state);
+}
+
 void QAndroidCaptureSession::setStatus(QMediaRecorder::Status status)
 {
     if (m_status == status)
@@ -503,7 +547,7 @@ void QAndroidCaptureSession::restartViewfinder()
 void QAndroidCaptureSession::updateDuration()
 {
     if (m_elapsedTime.isValid())
-        m_duration = m_elapsedTime.elapsed();
+        m_duration = m_elapsedTime.elapsed() + m_previousElapsedTime;
 
     emit durationChanged(m_duration);
 }
