@@ -42,6 +42,7 @@
 #include "avfvideooutput.h"
 
 #include <qpointer.h>
+#include <QFileInfo>
 
 #import <AVFoundation/AVFoundation.h>
 
@@ -66,16 +67,7 @@ static void *AVFMediaPlayerSessionObserverBufferLikelyToKeepUpContext = &AVFMedi
 static void *AVFMediaPlayerSessionObserverCurrentItemObservationContext = &AVFMediaPlayerSessionObserverCurrentItemObservationContext;
 static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext = &AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext;
 
-@interface AVFMediaPlayerSessionObserver : NSObject
-{
-@private
-    AVFMediaPlayerSession *m_session;
-    AVPlayer *m_player;
-    AVPlayerItem *m_playerItem;
-    AVPlayerLayer *m_playerLayer;
-    NSURL *m_URL;
-    BOOL m_bufferIsLikelyToKeepUp;
-}
+@interface AVFMediaPlayerSessionObserver : NSObject<AVAssetResourceLoaderDelegate>
 
 @property (readonly, getter=player) AVPlayer* m_player;
 @property (readonly, getter=playerItem) AVPlayerItem* m_playerItem;
@@ -83,7 +75,7 @@ static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext
 @property (readonly, getter=session) AVFMediaPlayerSession* m_session;
 
 - (AVFMediaPlayerSessionObserver *) initWithMediaPlayerSession:(AVFMediaPlayerSession *)session;
-- (void) setURL:(NSURL *)url;
+- (void) setURL:(NSURL *)url mimeType:(NSString *)mimeType;
 - (void) unloadMedia;
 - (void) prepareToPlayAsset:(AVURLAsset *)asset withKeys:(NSArray *)requestedKeys;
 - (void) assetFailedToPrepareForPlayback:(NSError *)error;
@@ -93,9 +85,21 @@ static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext
                          change:(NSDictionary *)change context:(void *)context;
 - (void) detatchSession;
 - (void) dealloc;
+- (BOOL) resourceLoader:(AVAssetResourceLoader *)resourceLoader shouldWaitForLoadingOfRequestedResource:(AVAssetResourceLoadingRequest *)loadingRequest;
 @end
 
 @implementation AVFMediaPlayerSessionObserver
+{
+@private
+    AVFMediaPlayerSession *m_session;
+    AVPlayer *m_player;
+    AVPlayerItem *m_playerItem;
+    AVPlayerLayer *m_playerLayer;
+    NSURL *m_URL;
+    BOOL m_bufferIsLikelyToKeepUp;
+    NSData *m_data;
+    NSString *m_mimeType;
+}
 
 @synthesize m_player, m_playerItem, m_playerLayer, m_session;
 
@@ -106,11 +110,20 @@ static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext
 
     self->m_session = session;
     self->m_bufferIsLikelyToKeepUp = FALSE;
+
+    m_playerLayer = [AVPlayerLayer playerLayerWithPlayer:nil];
+    [m_playerLayer retain];
+    m_playerLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
+    m_playerLayer.anchorPoint = CGPointMake(0.0f, 0.0f);
+
     return self;
 }
 
-- (void) setURL:(NSURL *)url
+- (void) setURL:(NSURL *)url mimeType:(NSString *)mimeType
 {
+    [m_mimeType release];
+    m_mimeType = [mimeType retain];
+
     if (m_URL != url)
     {
         [m_URL release];
@@ -122,6 +135,8 @@ static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext
         // use __block to avoid maintaining strong references on variables captured by the
         // following block callback
         __block AVURLAsset *asset = [[AVURLAsset URLAssetWithURL:m_URL options:nil] retain];
+        [asset.resourceLoader setDelegate:self queue:dispatch_get_main_queue()];
+
         __block NSArray *requestedKeys = [[NSArray arrayWithObjects:AVF_TRACKS_KEY, AVF_PLAYABLE_KEY, nil] retain];
 
         __block AVFMediaPlayerSessionObserver *blockSelf = self;
@@ -153,6 +168,10 @@ static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext
         [[NSNotificationCenter defaultCenter] removeObserver:self
                                                     name:AVPlayerItemTimeJumpedNotification
                                                     object:m_playerItem];
+        for (AVPlayerItemOutput *output in m_playerItem.outputs) {
+            if ([output isKindOfClass:[AVPlayerItemVideoOutput class]])
+                [m_playerItem removeOutput:output];
+        }
         m_playerItem = 0;
     }
     if (m_player) {
@@ -163,10 +182,6 @@ static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext
         [m_player release];
         m_player = 0;
     }
-    if (m_playerLayer) {
-        [m_playerLayer release];
-        m_playerLayer = 0;
-    }
 }
 
 - (void) prepareToPlayAsset:(AVURLAsset *)asset
@@ -251,14 +266,8 @@ static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext
         [m_player setMuted:m_session->isMuted()];
     }
 
-    //Create a new player layer if we don't have one already
-    if (!m_playerLayer)
-    {
-        m_playerLayer = [AVPlayerLayer playerLayerWithPlayer:m_player];
-        [m_playerLayer retain];
-        m_playerLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
-        m_playerLayer.anchorPoint = CGPointMake(0.0f, 0.0f);
-    }
+    //Assign the output layer to the new player
+    m_playerLayer.player = m_player;
 
     //Observe the AVPlayer "currentItem" property to find out when any
     //AVPlayer replaceCurrentItemWithPlayerItem: replacement will/did
@@ -339,7 +348,7 @@ static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext
 
             case AVPlayerStatusFailed:
             {
-                AVPlayerItem *playerItem = (AVPlayerItem *)object;
+                AVPlayerItem *playerItem = static_cast<AVPlayerItem*>(object);
                 [self assetFailedToPrepareForPlayback:playerItem.error];
 
                 if (self.session)
@@ -403,18 +412,58 @@ static void *AVFMediaPlayerSessionObserverCurrentItemDurationObservationContext
         [m_URL release];
     }
 
+    [m_mimeType release];
+    [m_playerLayer release];
     [super dealloc];
 }
 
+- (BOOL) resourceLoader:(AVAssetResourceLoader *)resourceLoader shouldWaitForLoadingOfRequestedResource:(AVAssetResourceLoadingRequest *)loadingRequest
+{
+    Q_UNUSED(resourceLoader);
+
+    if (![loadingRequest.request.URL.scheme isEqualToString:@"iodevice"])
+        return NO;
+
+    QIODevice *device = m_session->mediaStream();
+    if (!device)
+        return NO;
+
+    device->seek(loadingRequest.dataRequest.requestedOffset);
+    if (loadingRequest.contentInformationRequest) {
+        loadingRequest.contentInformationRequest.contentType = m_mimeType;
+        loadingRequest.contentInformationRequest.contentLength = device->size();
+        loadingRequest.contentInformationRequest.byteRangeAccessSupported = YES;
+    }
+
+    if (loadingRequest.dataRequest) {
+        NSInteger requestedLength = loadingRequest.dataRequest.requestedLength;
+        int maxBytes = qMin(32 * 1064, int(requestedLength));
+        char buffer[maxBytes];
+        NSInteger submitted = 0;
+        while (submitted < requestedLength) {
+            qint64 len = device->read(buffer, maxBytes);
+            if (len < 1)
+                break;
+
+            [loadingRequest.dataRequest respondWithData:[NSData dataWithBytes:buffer length:len]];
+            submitted += len;
+        }
+
+        // Finish loading even if not all bytes submitted.
+        [loadingRequest finishLoading];
+    }
+
+    return YES;
+}
 @end
 
 AVFMediaPlayerSession::AVFMediaPlayerSession(AVFMediaPlayerService *service, QObject *parent)
     : QObject(parent)
     , m_service(service)
-    , m_videoOutput(0)
+    , m_videoOutput(nullptr)
     , m_state(QMediaPlayer::StoppedState)
     , m_mediaStatus(QMediaPlayer::NoMedia)
-    , m_mediaStream(0)
+    , m_mediaStream(nullptr)
     , m_muted(false)
     , m_tryingAsync(false)
     , m_volume(100)
@@ -435,8 +484,8 @@ AVFMediaPlayerSession::~AVFMediaPlayerSession()
     qDebug() << Q_FUNC_INFO;
 #endif
     //Detatch the session from the sessionObserver (which could still be alive trying to communicate with this session).
-    [(AVFMediaPlayerSessionObserver*)m_observer detatchSession];
-    [(AVFMediaPlayerSessionObserver*)m_observer release];
+    [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) detatchSession];
+    [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) release];
 }
 
 void AVFMediaPlayerSession::setVideoOutput(AVFVideoOutput *output)
@@ -450,13 +499,13 @@ void AVFMediaPlayerSession::setVideoOutput(AVFVideoOutput *output)
 
     //Set the current output layer to null to stop rendering
     if (m_videoOutput) {
-        m_videoOutput->setLayer(0);
+        m_videoOutput->setLayer(nullptr);
     }
 
     m_videoOutput = output;
 
     if (m_videoOutput && m_state != QMediaPlayer::StoppedState)
-        m_videoOutput->setLayer([(AVFMediaPlayerSessionObserver*)m_observer playerLayer]);
+        m_videoOutput->setLayer([static_cast<AVFMediaPlayerSessionObserver*>(m_observer) playerLayer]);
 }
 
 void *AVFMediaPlayerSession::currentAssetHandle()
@@ -464,7 +513,7 @@ void *AVFMediaPlayerSession::currentAssetHandle()
 #ifdef QT_DEBUG_AVF
     qDebug() << Q_FUNC_INFO;
 #endif
-    AVAsset *currentAsset = [[(AVFMediaPlayerSessionObserver*)m_observer playerItem] asset];
+    AVAsset *currentAsset = [[static_cast<AVFMediaPlayerSessionObserver*>(m_observer) playerItem] asset];
     return currentAsset;
 }
 
@@ -483,21 +532,33 @@ QMediaContent AVFMediaPlayerSession::media() const
     return m_resources;
 }
 
-const QIODevice *AVFMediaPlayerSession::mediaStream() const
+QIODevice *AVFMediaPlayerSession::mediaStream() const
 {
     return m_mediaStream;
 }
 
+static void setURL(void *observer, const QByteArray &url, const QString &mimeType = QString())
+{
+    NSString *urlString = [NSString stringWithUTF8String:url.constData()];
+    NSURL *nsurl = [NSURL URLWithString:urlString];
+    [static_cast<AVFMediaPlayerSessionObserver*>(observer) setURL:nsurl mimeType:[NSString stringWithUTF8String:mimeType.toLatin1().constData()]];
+}
+
+static void setStreamURL(void *observer, const QByteArray &url)
+{
+    setURL(observer, QByteArrayLiteral("iodevice://") + url, QFileInfo(url).suffix());
+}
+
 void AVFMediaPlayerSession::setMedia(const QMediaContent &content, QIODevice *stream)
 {
 #ifdef QT_DEBUG_AVF
-    qDebug() << Q_FUNC_INFO << content.canonicalUrl();
+    qDebug() << Q_FUNC_INFO << content.request().url();
 #endif
 
-    [(AVFMediaPlayerSessionObserver*)m_observer unloadMedia];
+    [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) unloadMedia];
 
     m_resources = content;
-    m_mediaStream = stream;
+    resetStream(stream);
 
     setAudioAvailable(false);
     setVideoAvailable(false);
@@ -508,7 +569,7 @@ void AVFMediaPlayerSession::setMedia(const QMediaContent &content, QIODevice *st
     const QMediaPlayer::MediaStatus oldMediaStatus = m_mediaStatus;
     const QMediaPlayer::State oldState = m_state;
 
-    if (content.isNull() || content.canonicalUrl().isEmpty()) {
+    if (!m_mediaStream && (content.isNull() || content.request().url().isEmpty())) {
         m_mediaStatus = QMediaPlayer::NoMedia;
         if (m_mediaStatus != oldMediaStatus)
             Q_EMIT mediaStatusChanged(m_mediaStatus);
@@ -524,11 +585,16 @@ void AVFMediaPlayerSession::setMedia(const QMediaContent &content, QIODevice *st
     if (m_mediaStatus != oldMediaStatus)
         Q_EMIT mediaStatusChanged(m_mediaStatus);
 
-    //Load AVURLAsset
-    //initialize asset using content's URL
-    NSString *urlString = [NSString stringWithUTF8String:content.canonicalUrl().toEncoded().constData()];
-    NSURL *url = [NSURL URLWithString:urlString];
-    [(AVFMediaPlayerSessionObserver*)m_observer setURL:url];
+    if (m_mediaStream) {
+        // If there is a data, try to load it,
+        // otherwise wait for readyRead.
+        if (m_mediaStream->size())
+            setStreamURL(m_observer, m_resources.request().url().toEncoded());
+    } else {
+        //Load AVURLAsset
+        //initialize asset using content's URL
+        setURL(m_observer, m_resources.request().url().toEncoded());
+    }
 
     m_state = QMediaPlayer::StoppedState;
     if (m_state != oldState)
@@ -537,7 +603,7 @@ void AVFMediaPlayerSession::setMedia(const QMediaContent &content, QIODevice *st
 
 qint64 AVFMediaPlayerSession::position() const
 {
-    AVPlayerItem *playerItem = [(AVFMediaPlayerSessionObserver*)m_observer playerItem];
+    AVPlayerItem *playerItem = [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) playerItem];
 
     if (!playerItem)
         return m_requestedPosition != -1 ? m_requestedPosition : 0;
@@ -616,7 +682,7 @@ void AVFMediaPlayerSession::setSeekable(bool seekable)
 
 QMediaTimeRange AVFMediaPlayerSession::availablePlaybackRanges() const
 {
-    AVPlayerItem *playerItem = [(AVFMediaPlayerSessionObserver*)m_observer playerItem];
+    AVPlayerItem *playerItem = [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) playerItem];
 
     if (playerItem) {
         QMediaTimeRange timeRanges;
@@ -649,7 +715,7 @@ void AVFMediaPlayerSession::setPlaybackRate(qreal rate)
 
     m_rate = rate;
 
-    AVPlayer *player = [(AVFMediaPlayerSessionObserver*)m_observer player];
+    AVPlayer *player = [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player];
     if (player && m_state == QMediaPlayer::PlayingState)
         [player setRate:m_rate];
 
@@ -665,7 +731,7 @@ void AVFMediaPlayerSession::setPosition(qint64 pos)
     if (pos == position())
         return;
 
-    AVPlayerItem *playerItem = [(AVFMediaPlayerSessionObserver*)m_observer playerItem];
+    AVPlayerItem *playerItem = [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) playerItem];
     if (!playerItem) {
         m_requestedPosition = pos;
         Q_EMIT positionChanged(m_requestedPosition);
@@ -686,7 +752,7 @@ void AVFMediaPlayerSession::setPosition(qint64 pos)
 
     CMTime newTime = [playerItem currentTime];
     newTime.value = (pos / 1000.0f) * newTime.timescale;
-    [playerItem seekToTime:newTime];
+    [playerItem seekToTime:newTime toleranceBefore:kCMTimeZero toleranceAfter:kCMTimeZero completionHandler:nil];
 
     Q_EMIT positionChanged(pos);
 
@@ -711,7 +777,7 @@ void AVFMediaPlayerSession::play()
         return;
 
     if (m_videoOutput) {
-        m_videoOutput->setLayer([(AVFMediaPlayerSessionObserver*)m_observer playerLayer]);
+        m_videoOutput->setLayer([static_cast<AVFMediaPlayerSessionObserver*>(m_observer) playerLayer]);
     }
 
     // Reset media status if the current status is EndOfMedia
@@ -720,7 +786,7 @@ void AVFMediaPlayerSession::play()
 
     if (m_mediaStatus == QMediaPlayer::LoadedMedia || m_mediaStatus == QMediaPlayer::BufferedMedia) {
         // Setting the rate starts playback
-        [[(AVFMediaPlayerSessionObserver*)m_observer player] setRate:m_rate];
+        [[static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player] setRate:m_rate];
     }
 
     m_state = QMediaPlayer::PlayingState;
@@ -744,10 +810,10 @@ void AVFMediaPlayerSession::pause()
     m_state = QMediaPlayer::PausedState;
 
     if (m_videoOutput) {
-        m_videoOutput->setLayer([(AVFMediaPlayerSessionObserver*)m_observer playerLayer]);
+        m_videoOutput->setLayer([static_cast<AVFMediaPlayerSessionObserver*>(m_observer) playerLayer]);
     }
 
-    [[(AVFMediaPlayerSessionObserver*)m_observer player] pause];
+    [[static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player] pause];
 
     // Reset media status if the current status is EndOfMedia
     if (m_mediaStatus == QMediaPlayer::EndOfMedia)
@@ -767,11 +833,11 @@ void AVFMediaPlayerSession::stop()
         return;
 
     // AVPlayer doesn't have stop(), only pause() and play().
-    [[(AVFMediaPlayerSessionObserver*)m_observer player] pause];
+    [[static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player] pause];
     setPosition(0);
 
     if (m_videoOutput) {
-        m_videoOutput->setLayer(0);
+        m_videoOutput->setLayer(nullptr);
     }
 
     if (m_mediaStatus == QMediaPlayer::BufferedMedia)
@@ -792,7 +858,7 @@ void AVFMediaPlayerSession::setVolume(int volume)
 
     m_volume = volume;
 
-    AVPlayer *player = [(AVFMediaPlayerSessionObserver*)m_observer player];
+    AVPlayer *player = [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player];
     if (player)
         [player setVolume:volume / 100.0f];
 
@@ -810,7 +876,7 @@ void AVFMediaPlayerSession::setMuted(bool muted)
 
     m_muted = muted;
 
-    AVPlayer *player = [(AVFMediaPlayerSessionObserver*)m_observer player];
+    AVPlayer *player = [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player];
     if (player)
         [player setMuted:muted];
 
@@ -825,21 +891,20 @@ void AVFMediaPlayerSession::processEOS()
 #endif
     Q_EMIT positionChanged(position());
     m_mediaStatus = QMediaPlayer::EndOfMedia;
-    Q_EMIT mediaStatusChanged(m_mediaStatus);
-
     m_state = QMediaPlayer::StoppedState;
 
     // At this point, frames should not be rendered anymore.
     // Clear the output layer to make sure of that.
     if (m_videoOutput)
-        m_videoOutput->setLayer(0);
+        m_videoOutput->setLayer(nullptr);
 
+    Q_EMIT mediaStatusChanged(m_mediaStatus);
     Q_EMIT stateChanged(m_state);
 }
 
 void AVFMediaPlayerSession::processLoadStateChange(QMediaPlayer::State newState)
 {
-    AVPlayerStatus currentStatus = [[(AVFMediaPlayerSessionObserver*)m_observer player] status];
+    AVPlayerStatus currentStatus = [[static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player] status];
 
 #ifdef QT_DEBUG_AVF
     qDebug() << Q_FUNC_INFO << currentStatus << ", " << m_mediaStatus << ", " << newState;
@@ -852,7 +917,7 @@ void AVFMediaPlayerSession::processLoadStateChange(QMediaPlayer::State newState)
 
         QMediaPlayer::MediaStatus newStatus = m_mediaStatus;
 
-        AVPlayerItem *playerItem = [(AVFMediaPlayerSessionObserver*)m_observer playerItem];
+        AVPlayerItem *playerItem = [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) playerItem];
 
         if (playerItem) {
             // Check each track for audio and video content
@@ -874,11 +939,13 @@ void AVFMediaPlayerSession::processLoadStateChange(QMediaPlayer::State newState)
             setSeekable([[playerItem seekableTimeRanges] count] > 0);
 
             // Get the native size of the video, and reset the bounds of the player layer
-            AVPlayerLayer *playerLayer = [(AVFMediaPlayerSessionObserver*)m_observer playerLayer];
+            AVPlayerLayer *playerLayer = [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) playerLayer];
             if (videoTrack && playerLayer) {
-                playerLayer.bounds = CGRectMake(0.0f, 0.0f,
-                                                videoTrack.naturalSize.width,
-                                                videoTrack.naturalSize.height);
+                if (!playerLayer.bounds.size.width || !playerLayer.bounds.size.height) {
+                    playerLayer.bounds = CGRectMake(0.0f, 0.0f,
+                                                    videoTrack.naturalSize.width,
+                                                    videoTrack.naturalSize.height);
+                }
 
                 if (m_videoOutput && newState != QMediaPlayer::StoppedState) {
                     m_videoOutput->setLayer(playerLayer);
@@ -899,9 +966,9 @@ void AVFMediaPlayerSession::processLoadStateChange(QMediaPlayer::State newState)
 
     }
 
-    if (newState == QMediaPlayer::PlayingState && [(AVFMediaPlayerSessionObserver*)m_observer player]) {
+    if (newState == QMediaPlayer::PlayingState && [static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player]) {
         // Setting the rate is enough to start playback, no need to call play()
-        [[(AVFMediaPlayerSessionObserver*)m_observer player] setRate:m_rate];
+        [[static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player] setRate:m_rate];
     }
 }
 
@@ -922,6 +989,20 @@ void AVFMediaPlayerSession::processBufferStateChange(int bufferStatus)
     if (bufferStatus == m_bufferStatus)
         return;
 
+    auto status = m_mediaStatus;
+    // Buffered -> unbuffered.
+    if (!bufferStatus) {
+        status = QMediaPlayer::StalledMedia;
+    } else if (status == QMediaPlayer::StalledMedia) {
+        status = QMediaPlayer::BufferedMedia;
+        // Resume playback.
+        if (m_state == QMediaPlayer::PlayingState)
+            [[static_cast<AVFMediaPlayerSessionObserver*>(m_observer) player] setRate:m_rate];
+    }
+
+    if (m_mediaStatus != status)
+        Q_EMIT mediaStatusChanged(m_mediaStatus = status);
+
     m_bufferStatus = bufferStatus;
     Q_EMIT bufferStatusChanged(bufferStatus);
 }
@@ -954,3 +1035,28 @@ void AVFMediaPlayerSession::processMediaLoadError()
 
     Q_EMIT error(QMediaPlayer::FormatError, tr("Failed to load media"));
 }
+
+void AVFMediaPlayerSession::streamReady()
+{
+    setStreamURL(m_observer, m_resources.request().url().toEncoded());
+}
+
+void AVFMediaPlayerSession::streamDestroyed()
+{
+    resetStream(nullptr);
+}
+
+void AVFMediaPlayerSession::resetStream(QIODevice *stream)
+{
+    if (m_mediaStream) {
+        disconnect(m_mediaStream, &QIODevice::readyRead, this, &AVFMediaPlayerSession::streamReady);
+        disconnect(m_mediaStream, &QIODevice::destroyed, this, &AVFMediaPlayerSession::streamDestroyed);
+    }
+
+    m_mediaStream = stream;
+
+    if (m_mediaStream) {
+        connect(m_mediaStream, &QIODevice::readyRead, this, &AVFMediaPlayerSession::streamReady);
+        connect(m_mediaStream, &QIODevice::destroyed, this, &AVFMediaPlayerSession::streamDestroyed);
+    }
+}
