@@ -50,6 +50,9 @@
 #include <qcoreapplication.h>
 #include <qmath.h>
 #include <QtCore/qdebug.h>
+
+#include <mutex>
+
 #include <float.h>
 #include <evcode.h>
 
@@ -542,7 +545,6 @@ EVRCustomPresenter::EVRCustomPresenter(QAbstractVideoSurface *surface)
     , m_sampleFreeCB(this, &EVRCustomPresenter::onSampleFree)
     , m_refCount(1)
     , m_renderState(RenderShutdown)
-    , m_mutex(QMutex::Recursive)
     , m_scheduler(this)
     , m_tokenCounter(0)
     , m_sampleNotify(false)
@@ -659,7 +661,7 @@ HRESULT EVRCustomPresenter::InitServicePointers(IMFTopologyServiceLookup *lookup
     HRESULT hr = S_OK;
     DWORD objectCount = 0;
 
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     // Do not allow initializing when playing or paused.
     if (isActive())
@@ -739,7 +741,7 @@ HRESULT EVRCustomPresenter::ProcessMessage(MFVP_MESSAGE_TYPE message, ULONG_PTR
 {
     HRESULT hr = S_OK;
 
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     hr = checkShutdown();
     if (FAILED(hr))
@@ -806,7 +808,7 @@ HRESULT EVRCustomPresenter::GetCurrentMediaType(IMFVideoMediaType **mediaType)
 
     *mediaType = NULL;
 
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     hr = checkShutdown();
     if (FAILED(hr))
@@ -820,7 +822,7 @@ HRESULT EVRCustomPresenter::GetCurrentMediaType(IMFVideoMediaType **mediaType)
 
 HRESULT EVRCustomPresenter::OnClockStart(MFTIME, LONGLONG clockStartOffset)
 {
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     // We cannot start after shutdown.
     HRESULT hr = checkShutdown();
@@ -855,7 +857,7 @@ HRESULT EVRCustomPresenter::OnClockStart(MFTIME, LONGLONG clockStartOffset)
 
 HRESULT EVRCustomPresenter::OnClockRestart(MFTIME)
 {
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     HRESULT hr = checkShutdown();
     if (FAILED(hr))
@@ -879,7 +881,7 @@ HRESULT EVRCustomPresenter::OnClockRestart(MFTIME)
 
 HRESULT EVRCustomPresenter::OnClockStop(MFTIME)
 {
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     HRESULT hr = checkShutdown();
     if (FAILED(hr))
@@ -899,7 +901,7 @@ HRESULT EVRCustomPresenter::OnClockStop(MFTIME)
 
 HRESULT EVRCustomPresenter::OnClockPause(MFTIME)
 {
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     // We cannot pause the clock after shutdown.
     HRESULT hr = checkShutdown();
@@ -916,7 +918,7 @@ HRESULT EVRCustomPresenter::OnClockSetRate(MFTIME, float rate)
     // The presenter reports its maximum rate through the IMFRateSupport interface.
     // Here, we assume that the EVR honors the maximum rate.
 
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     HRESULT hr = checkShutdown();
     if (FAILED(hr))
@@ -944,7 +946,7 @@ HRESULT EVRCustomPresenter::GetSlowestRate(MFRATE_DIRECTION, BOOL, float *rate)
     if (!rate)
         return E_POINTER;
 
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     HRESULT hr = checkShutdown();
 
@@ -961,7 +963,7 @@ HRESULT EVRCustomPresenter::GetFastestRate(MFRATE_DIRECTION direction, BOOL thin
     if (!rate)
         return E_POINTER;
 
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     float maxRate = 0.0f;
 
@@ -983,7 +985,7 @@ HRESULT EVRCustomPresenter::GetFastestRate(MFRATE_DIRECTION direction, BOOL thin
 
 HRESULT EVRCustomPresenter::IsRateSupported(BOOL thin, float rate, float *nearestSupportedRate)
 {
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     float maxRate = 0.0f;
     float nearestRate = rate;  // If we support rate, that is the nearest.
@@ -1017,7 +1019,7 @@ HRESULT EVRCustomPresenter::IsRateSupported(BOOL thin, float rate, float *neares
 
 void EVRCustomPresenter::supportedFormatsChanged()
 {
-    QMutexLocker locker(&m_mutex);
+    const std::lock_guard<QRecursiveMutex> locker(m_mutex);
 
     m_canRenderToSurface = false;
     m_presentEngine->setHint(D3DPresentEngine::RenderToTexture, false);
@@ -1030,7 +1032,7 @@ void EVRCustomPresenter::supportedFormatsChanged()
             m_canRenderToSurface = true;
         } else {
             formats = m_surface->supportedPixelFormats(QAbstractVideoBuffer::NoHandle);
-            Q_FOREACH (QVideoFrame::PixelFormat format, formats) {
+            for (QVideoFrame::PixelFormat format : qAsConst(formats)) {
                 if (SUCCEEDED(m_presentEngine->checkFormat(qt_evr_D3DFormatFromPixelFormat(format)))) {
                     m_canRenderToSurface = true;
                     break;
@@ -1063,6 +1065,13 @@ void EVRCustomPresenter::setSurface(QAbstractVideoSurface *surface)
     supportedFormatsChanged();
 }
 
+void EVRCustomPresenter::setCropRect(QRect cropRect)
+{
+    m_mutex.lock();
+    m_cropRect = cropRect;
+    m_mutex.unlock();
+}
+
 HRESULT EVRCustomPresenter::configureMixer(IMFTransform *mixer)
 {
     // Set the zoom rectangle (ie, the source clipping rectangle).
@@ -1209,6 +1218,8 @@ HRESULT EVRCustomPresenter::checkEndOfStream()
     // Everything is complete. Now we can tell the EVR that we are done.
     notifyEvent(EC_COMPLETE, (LONG_PTR)S_OK, 0);
     m_endStreaming = false;
+
+    stopSurface();
     return S_OK;
 }
 
@@ -1351,13 +1362,30 @@ HRESULT EVRCustomPresenter::createOptimalVideoType(IMFMediaType *proposedType, I
     hr = proposedType->GetUINT64(MF_MT_FRAME_SIZE, &size);
     width = int(HI32(size));
     height = int(LO32(size));
-    rcOutput.left = 0;
-    rcOutput.top = 0;
-    rcOutput.right = width;
-    rcOutput.bottom = height;
+
+    if (m_cropRect.isValid()) {
+        rcOutput.left = m_cropRect.x();
+        rcOutput.top = m_cropRect.y();
+        rcOutput.right = m_cropRect.x() + m_cropRect.width();
+        rcOutput.bottom = m_cropRect.y() + m_cropRect.height();
+
+        m_sourceRect.left = float(m_cropRect.x()) / width;
+        m_sourceRect.top = float(m_cropRect.y()) / height;
+        m_sourceRect.right = float(m_cropRect.x() + m_cropRect.width()) / width;
+        m_sourceRect.bottom = float(m_cropRect.y() + m_cropRect.height()) / height;
+
+        if (m_mixer)
+            configureMixer(m_mixer);
+    } else {
+        rcOutput.left = 0;
+        rcOutput.top = 0;
+        rcOutput.right = width;
+        rcOutput.bottom = height;
+    }
 
     // Set the geometric aperture, and disable pan/scan.
-    displayArea = qt_evr_makeMFArea(0, 0, rcOutput.right, rcOutput.bottom);
+    displayArea = qt_evr_makeMFArea(0, 0, rcOutput.right - rcOutput.left,
+                                    rcOutput.bottom - rcOutput.top);
 
     hr = mtOptimal->SetUINT32(MF_MT_PAN_SCAN_ENABLED, FALSE);
     if (FAILED(hr))
@@ -1423,7 +1451,7 @@ HRESULT EVRCustomPresenter::setMediaType(IMFMediaType *mediaType)
     // Initialize the presenter engine with the new media type.
     // The presenter engine allocates the samples.
 
-    hr = m_presentEngine->createVideoSamples(mediaType, sampleQueue);
+    hr = m_presentEngine->createVideoSamples(mediaType, sampleQueue, m_cropRect.size());
     if (FAILED(hr))
         goto done;
 
