@@ -123,36 +123,8 @@ private:
 
 DirectShowPlayerService::DirectShowPlayerService(QObject *parent)
     : QMediaService(parent)
-    , m_playerControl(0)
-    , m_metaDataControl(0)
-    , m_videoRendererControl(0)
-    , m_videoWindowControl(0)
-    , m_audioEndpointControl(0)
-    , m_audioProbeControl(nullptr)
-    , m_videoProbeControl(nullptr)
-    , m_audioSampleGrabber(nullptr)
-    , m_videoSampleGrabber(nullptr)
-    , m_taskThread(0)
     , m_loop(qt_directShowEventLoop())
-    , m_pendingTasks(0)
-    , m_executingTask(0)
-    , m_executedTasks(0)
     , m_taskHandle(::CreateEvent(nullptr, FALSE, FALSE, nullptr))
-    , m_eventHandle(0)
-    , m_graphStatus(NoMedia)
-    , m_stream(0)
-    , m_graph(0)
-    , m_source(0)
-    , m_audioOutput(0)
-    , m_videoOutput(0)
-    , m_rate(1.0)
-    , m_position(0)
-    , m_seekPosition(-1)
-    , m_duration(0)
-    , m_buffering(false)
-    , m_seekable(false)
-    , m_atEnd(false)
-    , m_dontCacheNextSeekResult(false)
 {
     m_playerControl = new DirectShowPlayerControl(this);
     m_metaDataControl = new DirectShowMetaDataControl(this);
@@ -178,12 +150,12 @@ DirectShowPlayerService::~DirectShowPlayerService()
 
     if (m_audioOutput) {
         m_audioOutput->Release();
-        m_audioOutput = 0;
+        m_audioOutput = nullptr;
     }
 
     if (m_videoOutput) {
         m_videoOutput->Release();
-        m_videoOutput = 0;
+        m_videoOutput = nullptr;
     }
 
     delete m_playerControl;
@@ -199,13 +171,13 @@ DirectShowPlayerService::~DirectShowPlayerService()
 
 QMediaControl *DirectShowPlayerService::requestControl(const char *name)
 {
-    if (qstrcmp(name, QMediaPlayerControl_iid) == 0) {
+    if (qstrcmp(name, QMediaPlayerControl_iid) == 0)
         return m_playerControl;
-    } else if (qstrcmp(name, QAudioOutputSelectorControl_iid) == 0) {
+    if (qstrcmp(name, QAudioOutputSelectorControl_iid) == 0)
         return m_audioEndpointControl;
-    } else if (qstrcmp(name, QMetaDataReaderControl_iid) == 0) {
+    if (qstrcmp(name, QMetaDataReaderControl_iid) == 0)
         return m_metaDataControl;
-    } else if (qstrcmp(name, QVideoRendererControl_iid) == 0) {
+    if (qstrcmp(name, QVideoRendererControl_iid) == 0) {
         if (!m_videoRendererControl && !m_videoWindowControl) {
             m_videoRendererControl = new DirectShowVideoRendererControl(m_loop);
 
@@ -214,7 +186,9 @@ QMediaControl *DirectShowPlayerService::requestControl(const char *name)
 
             return m_videoRendererControl;
         }
-    } else if (qstrcmp(name, QVideoWindowControl_iid) == 0) {
+        return nullptr;
+    }
+    if (qstrcmp(name, QVideoWindowControl_iid) == 0) {
         if (!m_videoRendererControl && !m_videoWindowControl) {
             IBaseFilter *filter{};
 
@@ -238,20 +212,23 @@ QMediaControl *DirectShowPlayerService::requestControl(const char *name)
 
             return m_videoWindowControl;
         }
-    } else if (qstrcmp(name, QMediaAudioProbeControl_iid) == 0) {
+        return nullptr;
+    }
+    if (qstrcmp(name, QMediaAudioProbeControl_iid) == 0) {
         if (!m_audioProbeControl)
             m_audioProbeControl = new DirectShowAudioProbeControl();
         m_audioProbeControl->ref();
         updateAudioProbe();
         return m_audioProbeControl;
-    } else if (qstrcmp(name, QMediaVideoProbeControl_iid) == 0) {
+    }
+    if (qstrcmp(name, QMediaVideoProbeControl_iid) == 0) {
         if (!m_videoProbeControl)
             m_videoProbeControl = new DirectShowVideoProbeControl();
         m_videoProbeControl->ref();
         updateVideoProbe();
         return m_videoProbeControl;
     }
-    return 0;
+    return nullptr;
 }
 
 void DirectShowPlayerService::releaseControl(QMediaControl *control)
@@ -260,17 +237,17 @@ void DirectShowPlayerService::releaseControl(QMediaControl *control)
         qWarning("QMediaService::releaseControl():"
                 " Attempted release of null control");
     } else if (control == m_videoRendererControl) {
-        setVideoOutput(0);
+        setVideoOutput(nullptr);
 
         delete m_videoRendererControl;
 
-        m_videoRendererControl = 0;
+        m_videoRendererControl = nullptr;
     } else if (control == m_videoWindowControl) {
-        setVideoOutput(0);
+        setVideoOutput(nullptr);
 
         delete m_videoWindowControl;
 
-        m_videoWindowControl = 0;
+        m_videoWindowControl = nullptr;
     } else if (control == m_audioProbeControl) {
         if (!m_audioProbeControl->deref()) {
             DirectShowAudioProbeControl *old = m_audioProbeControl;
@@ -297,7 +274,7 @@ void DirectShowPlayerService::load(const QMediaContent &media, QIODevice *stream
     if (m_graph)
         releaseGraph();
 
-    m_url = media.canonicalUrl();
+    m_url = media.request().url();
 
     m_stream = stream;
     m_error = QMediaPlayer::NoError;
@@ -311,7 +288,7 @@ void DirectShowPlayerService::load(const QMediaContent &media, QIODevice *stream
     m_seekable = false;
     m_atEnd = false;
     m_dontCacheNextSeekResult = false;
-    m_metaDataControl->reset();
+    m_metaDataControl->setMetadata(QVariantMap());
 
     if (m_url.isEmpty() && !stream) {
         m_pendingTasks = 0;
@@ -326,7 +303,17 @@ void DirectShowPlayerService::load(const QMediaContent &media, QIODevice *stream
             0x36b73882, 0xc2c8, 0x11cf, {0x8b, 0x46, 0x00, 0x80, 0x5f, 0x6c, 0xef, 0x60} };
         m_graphStatus = Loading;
 
+        DirectShowUtils::CoInitializeIfNeeded();
         m_graph = com_new<IFilterGraph2>(CLSID_FilterGraph, iid_IFilterGraph2);
+        m_graphBuilder = com_new<ICaptureGraphBuilder2>(CLSID_CaptureGraphBuilder2, IID_ICaptureGraphBuilder2);
+
+        // Attach the filter graph to the capture graph.
+        HRESULT hr = m_graphBuilder->SetFiltergraph(m_graph);
+        if (FAILED(hr)) {
+            qCWarning(qtDirectShowPlugin, "[0x%x] Failed to attach filter to capture graph", hr);
+            m_graphBuilder->Release();
+            m_graphBuilder = nullptr;
+        }
 
         if (stream)
             m_pendingTasks = SetStreamSource;
@@ -345,7 +332,7 @@ void DirectShowPlayerService::load(const QMediaContent &media, QIODevice *stream
 
 void DirectShowPlayerService::doSetUrlSource(QMutexLocker *locker)
 {
-    IBaseFilter *source = 0;
+    IBaseFilter *source = nullptr;
 
     HRESULT hr = E_FAIL;
     if (m_url.scheme() == QLatin1String("http") || m_url.scheme() == QLatin1String("https")) {
@@ -358,14 +345,14 @@ void DirectShowPlayerService::doSetUrlSource(QMutexLocker *locker)
 
         if (IFileSourceFilter *fileSource = com_new<IFileSourceFilter>(clsid_WMAsfReader, iid_IFileSourceFilter)) {
             locker->unlock();
-            hr = fileSource->Load(reinterpret_cast<const OLECHAR *>(m_url.toString().utf16()), 0);
+            hr = fileSource->Load(reinterpret_cast<const OLECHAR *>(m_url.toString().utf16()), nullptr);
 
             if (SUCCEEDED(hr)) {
                 source = com_cast<IBaseFilter>(fileSource, IID_IBaseFilter);
 
                 if (!SUCCEEDED(hr = m_graph->AddFilter(source, L"Source")) && source) {
                     source->Release();
-                    source = 0;
+                    source = nullptr;
                 }
             }
             fileSource->Release();
@@ -500,18 +487,18 @@ void DirectShowPlayerService::doRender(QMutexLocker *locker)
     HRESULT renderHr = S_OK;
 
     while (!filters.isEmpty()) {
-        IEnumPins *pins = 0;
+        IEnumPins *pins = nullptr;
         IBaseFilter *filter = filters[filters.size() - 1];
         filters.removeLast();
 
         if (!(m_pendingTasks & ReleaseFilters) && SUCCEEDED(filter->EnumPins(&pins))) {
             int outputs = 0;
-            for (IPin *pin = 0; pins->Next(1, &pin, 0) == S_OK; pin->Release()) {
+            for (IPin *pin = nullptr; pins->Next(1, &pin, nullptr) == S_OK; pin->Release()) {
                 PIN_DIRECTION direction;
                 if (pin->QueryDirection(&direction) == S_OK && direction == PINDIR_OUTPUT) {
                     ++outputs;
 
-                    IPin *peer = 0;
+                    IPin *peer = nullptr;
                     if (pin->ConnectedTo(&peer) == S_OK) {
                         PIN_INFO peerInfo;
                         if (SUCCEEDED(peer->QueryPinInfo(&peerInfo)))
@@ -519,7 +506,7 @@ void DirectShowPlayerService::doRender(QMutexLocker *locker)
                         peer->Release();
                     } else {
                         locker->unlock();
-                        HRESULT hr = graph->RenderEx(pin, /*AM_RENDEREX_RENDERTOEXISTINGRENDERERS*/ 1, 0);
+                        HRESULT hr = graph->RenderEx(pin, /*AM_RENDEREX_RENDERTOEXISTINGRENDERERS*/ 1, nullptr);
                         if (SUCCEEDED(hr)) {
                             rendered = true;
                             m_error = QMediaPlayer::NoError;
@@ -597,7 +584,6 @@ void DirectShowPlayerService::doRender(QMutexLocker *locker)
 
 void DirectShowPlayerService::doFinalizeLoad(QMutexLocker *locker)
 {
-    Q_UNUSED(locker)
     if (m_graphStatus != Loaded) {
         if (IMediaEvent *event = com_cast<IMediaEvent>(m_graph, IID_IMediaEvent)) {
             event->GetEventHandle(reinterpret_cast<OAEVENT *>(&m_eventHandle));
@@ -626,11 +612,19 @@ void DirectShowPlayerService::doFinalizeLoad(QMutexLocker *locker)
 
     m_graphStatus = Loaded;
 
+    // Do not block gui thread while updating metadata from file.
+    locker->unlock();
+    DirectShowMetaDataControl::updateMetadata(m_url.toString(), m_metadata);
+    locker->relock();
+
     QCoreApplication::postEvent(this, new QEvent(QEvent::Type(FinalizedLoad)));
 }
 
 void DirectShowPlayerService::releaseGraph()
 {
+    if (m_videoProbeControl)
+        m_videoProbeControl->flushVideoFrame();
+
     if (m_graph) {
         if (m_executingTask != 0) {
             // {8E1C39A1-DE53-11cf-AA63-0080C744528D}
@@ -650,6 +644,7 @@ void DirectShowPlayerService::releaseGraph()
         ::SetEvent(m_taskHandle);
 
         m_loop->wait(&m_mutex);
+        DirectShowUtils::CoUninitializeIfNeeded();
     }
 }
 
@@ -667,13 +662,18 @@ void DirectShowPlayerService::doReleaseGraph(QMutexLocker *locker)
 
     if (m_source) {
         m_source->Release();
-        m_source = 0;
+        m_source = nullptr;
     }
 
-    m_eventHandle = 0;
+    m_eventHandle = nullptr;
 
     m_graph->Release();
-    m_graph = 0;
+    m_graph = nullptr;
+
+    if (m_graphBuilder) {
+        m_graphBuilder->Release();
+        m_graphBuilder = nullptr;
+    }
 
     m_loop->wake();
 }
@@ -685,7 +685,7 @@ void DirectShowPlayerService::doSetVideoProbe(QMutexLocker *locker)
 {
     Q_UNUSED(locker);
 
-    if (!m_graph) {
+    if (!m_graph || !m_graphBuilder) {
         qCWarning(qtDirectShowPlugin, "Attempting to set a video probe without a valid graph!");
         return;
     }
@@ -701,41 +701,14 @@ void DirectShowPlayerService::doSetVideoProbe(QMutexLocker *locker)
         return;
     }
 
-    // TODO: Make util function for getting this, so it's easy to keep it in sync.
-    static const GUID subtypes[] = { MEDIASUBTYPE_ARGB32,
-                                     MEDIASUBTYPE_RGB32,
-                                     MEDIASUBTYPE_RGB24,
-                                     MEDIASUBTYPE_RGB565,
-                                     MEDIASUBTYPE_RGB555,
-                                     MEDIASUBTYPE_AYUV,
-                                     MEDIASUBTYPE_I420,
-                                     MEDIASUBTYPE_IYUV,
-                                     MEDIASUBTYPE_YV12,
-                                     MEDIASUBTYPE_UYVY,
-                                     MEDIASUBTYPE_YUYV,
-                                     MEDIASUBTYPE_YUY2,
-                                     MEDIASUBTYPE_NV12,
-                                     MEDIASUBTYPE_MJPG,
-                                     MEDIASUBTYPE_IMC1,
-                                     MEDIASUBTYPE_IMC2,
-                                     MEDIASUBTYPE_IMC3,
-                                     MEDIASUBTYPE_IMC4 };
+    DirectShowMediaType mediaType({ MEDIATYPE_Video, MEDIASUBTYPE_ARGB32 });
+    m_videoSampleGrabber->setMediaType(&mediaType);
 
-    // Negotiate the subtype
-    DirectShowMediaType mediaType(AM_MEDIA_TYPE { MEDIATYPE_Video });
-    const int items = (sizeof subtypes / sizeof(GUID));
-    bool connected = false;
-    for (int i = 0; i != items; ++i) {
-        mediaType->subtype = subtypes[i];
-        m_videoSampleGrabber->setMediaType(&mediaType);
-        if (DirectShowUtils::connectFilters(m_graph, m_source, m_videoSampleGrabber->filter(), true)) {
-            connected = true;
-            break;
-        }
-    }
-
-    if (!connected) {
-        qCWarning(qtDirectShowPlugin, "Unable to connect the video probe!");
+    // Connect source filter to sample grabber filter.
+    HRESULT hr = m_graphBuilder->RenderStream(nullptr, &MEDIATYPE_Video,
+                                              m_source, nullptr, m_videoSampleGrabber->filter());
+    if (FAILED(hr)) {
+        qCWarning(qtDirectShowPlugin, "[0x%x] Failed to connect the video sample grabber", hr);
         return;
     }
 
@@ -766,8 +739,15 @@ void DirectShowPlayerService::doSetAudioProbe(QMutexLocker *locker)
     }
 
     if (!DirectShowUtils::connectFilters(m_graph, m_source, m_audioSampleGrabber->filter(), true)) {
-        qCWarning(qtDirectShowPlugin, "Failed to connect the audio sample grabber");
-        return;
+        // Connect source filter to sample grabber filter.
+        HRESULT hr = m_graphBuilder
+            ? m_graphBuilder->RenderStream(nullptr, &MEDIATYPE_Audio,
+                                           m_source, nullptr, m_audioSampleGrabber->filter())
+            : E_FAIL;
+        if (FAILED(hr)) {
+            qCWarning(qtDirectShowPlugin, "[0x%x] Failed to connect the audio sample grabber", hr);
+            return;
+        }
     }
 
     m_audioSampleGrabber->start(DirectShowSampleGrabber::CallbackMethod::BufferCB);
@@ -826,17 +806,17 @@ int DirectShowPlayerService::findStreamTypes(IBaseFilter *source) const
     int streamTypes = 0;
 
     while (!filters.isEmpty()) {
-        IEnumPins *pins = 0;
+        IEnumPins *pins = nullptr;
         IBaseFilter *filter = filters[filters.size() - 1];
         filters.removeLast();
 
         if (SUCCEEDED(filter->EnumPins(&pins))) {
-            for (IPin *pin = 0; pins->Next(1, &pin, 0) == S_OK; pin->Release()) {
+            for (IPin *pin = nullptr; pins->Next(1, &pin, nullptr) == S_OK; pin->Release()) {
                 PIN_DIRECTION direction;
                 if (pin->QueryDirection(&direction) == S_OK && direction == PINDIR_OUTPUT) {
                     DirectShowMediaType connectionType;
                     if (SUCCEEDED(pin->ConnectionMediaType(&connectionType))) {
-                        IPin *peer = 0;
+                        IPin *peer = nullptr;
 
                         if (connectionType->majortype == MEDIATYPE_Audio) {
                             streamTypes |= AudioStream;
@@ -869,8 +849,8 @@ int DirectShowPlayerService::findStreamType(IPin *pin) const
         bool audio = false;
         bool other = false;
 
-        for (AM_MEDIA_TYPE *type = 0;
-                types->Next(1, &type, 0) == S_OK;
+        for (AM_MEDIA_TYPE *type = nullptr;
+                types->Next(1, &type, nullptr) == S_OK;
                 DirectShowMediaType::deleteType(type)) {
             if (type->majortype == MEDIATYPE_Audio)
                 audio = true;
@@ -1022,6 +1002,8 @@ void DirectShowPlayerService::stop()
 
     if ((m_executingTask | m_executedTasks) & (Play | Pause | Seek)) {
         m_pendingTasks |= Stop;
+        if (m_videoProbeControl)
+            m_videoProbeControl->flushVideoFrame();
 
         ::SetEvent(m_taskHandle);
 
@@ -1182,7 +1164,7 @@ void DirectShowPlayerService::doSeek(QMutexLocker *locker)
 
         locker->unlock();
         seeking->SetPositions(
-                &seekPosition, AM_SEEKING_AbsolutePositioning, 0, AM_SEEKING_NoPositioning);
+                &seekPosition, AM_SEEKING_AbsolutePositioning, nullptr, AM_SEEKING_NoPositioning);
         locker->relock();
 
         if (!m_dontCacheNextSeekResult) {
@@ -1208,7 +1190,7 @@ int DirectShowPlayerService::bufferStatus() const
             m_source, IID_IWMReaderAdvanced2)) {
         DWORD percentage = 0;
 
-        reader->GetBufferProgress(&percentage, 0);
+        reader->GetBufferProgress(&percentage, nullptr);
         reader->Release();
 
         return percentage;
@@ -1381,7 +1363,7 @@ void DirectShowPlayerService::doReleaseVideoOutput(QMutexLocker *locker)
         control->Release();
     }
 
-    IBaseFilter *intermediate = 0;
+    IBaseFilter *intermediate = nullptr;
     if (!SUCCEEDED(m_graph->FindFilterByName(L"Color Space Converter", &intermediate))) {
         intermediate = m_videoOutput;
         intermediate->AddRef();
@@ -1418,7 +1400,11 @@ void DirectShowPlayerService::customEvent(QEvent *event)
         QMutexLocker locker(&m_mutex);
 
         m_playerControl->updateMediaInfo(m_duration, m_streamTypes, m_seekable);
-        m_metaDataControl->updateMetadata(m_graph, m_source, m_url.toString());
+        if (m_metadata.isEmpty())
+            DirectShowMetaDataControl::updateMetadata(m_graph, m_source, m_metadata);
+
+        m_metaDataControl->setMetadata(m_metadata);
+        m_metadata.clear();
 
         updateStatus();
     } else if (event->type() == QEvent::Type(Error)) {
@@ -1450,6 +1436,8 @@ void DirectShowPlayerService::customEvent(QEvent *event)
             m_playerControl->updateState(QMediaPlayer::StoppedState);
             m_playerControl->updateStatus(QMediaPlayer::EndOfMedia);
             m_playerControl->updatePosition(m_position);
+            if (m_videoProbeControl)
+                m_videoProbeControl->flushVideoFrame();
         }
     } else if (event->type() == QEvent::Type(PositionChange)) {
         QMutexLocker locker(&m_mutex);
@@ -1558,7 +1546,7 @@ void DirectShowPlayerService::onVideoBufferAvailable(double time, const QByteArr
                       size,
                       format);
 
-    Q_EMIT m_videoProbeControl->videoFrameProbed(frame);
+    m_videoProbeControl->probeVideoFrame(frame);
 }
 
 QT_WARNING_POP
@@ -1652,13 +1640,13 @@ bool DirectShowPlayerService::isConnected(IBaseFilter *filter, PIN_DIRECTION dir
 {
     bool connected = false;
 
-    IEnumPins *pins = 0;
+    IEnumPins *pins = nullptr;
 
     if (SUCCEEDED(filter->EnumPins(&pins))) {
-        for (IPin *pin = 0; pins->Next(1, &pin, 0) == S_OK; pin->Release()) {
+        for (IPin *pin = nullptr; pins->Next(1, &pin, nullptr) == S_OK; pin->Release()) {
             PIN_DIRECTION dir;
             if (SUCCEEDED(pin->QueryDirection(&dir)) && dir == direction) {
-                IPin *peer = 0;
+                IPin *peer = nullptr;
                 if (SUCCEEDED(pin->ConnectedTo(&peer))) {
                     connected = true;
 
@@ -1674,15 +1662,15 @@ bool DirectShowPlayerService::isConnected(IBaseFilter *filter, PIN_DIRECTION dir
 IBaseFilter *DirectShowPlayerService::getConnected(
         IBaseFilter *filter, PIN_DIRECTION direction) const
 {
-    IBaseFilter *connected = 0;
+    IBaseFilter *connected = nullptr;
 
-    IEnumPins *pins = 0;
+    IEnumPins *pins = nullptr;
 
     if (SUCCEEDED(filter->EnumPins(&pins))) {
-        for (IPin *pin = 0; pins->Next(1, &pin, 0) == S_OK; pin->Release()) {
+        for (IPin *pin = nullptr; pins->Next(1, &pin, nullptr) == S_OK; pin->Release()) {
             PIN_DIRECTION dir;
             if (SUCCEEDED(pin->QueryDirection(&dir)) && dir == direction) {
-                IPin *peer = 0;
+                IPin *peer = nullptr;
                 if (SUCCEEDED(pin->ConnectedTo(&peer))) {
                     PIN_INFO info;
 
